/*
    buffer.c - functions for buffers
    Copyright (C) 2008 Telekatz <telekatz@gmx.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/


/* =================================== Includes ============================ */


#include <string.h>
#include "buffer.h"

// #ifndef WIN32

#include "irq.h"


/* ================== Datentypen und Variablendefinitionen ================= */

// #else
// int disableIRQ () {return 1;}
// #define restoreIRQ(x)
// #endif   // #else #ifndef WIN32


/* ======================= Funktionen und Programmcode ===================== */


/*
   ----------------------------------------------------------------------------
   Funktion: cBufferInit

   Initialisiert den Buffer mit Zeiger auf Buffer-Strukt,
   Pointer auf Speicherbereich und Groesse des Buffers

   ----------------------------------------------------------------------------
*/
void cBufferInit (struct cBuffer_ *buffer, unsigned char *start, unsigned short size)
{

   /* ----- lokale Variablen ----- */
   unsigned cpsr;


   /* ----- Programmcode ----- */
   cpsr = disableIRQ ();

   buffer->data = start;
   buffer->size = size;
   buffer->len = 0;
   buffer->index = 0;

   restoreIRQ (cpsr);
}
/* ----- Ende cBufferInit -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cBufferFlush

   Setzt den Buffer mit Laenge und Index zurueck auf NULL

   ----------------------------------------------------------------------------
*/
void cBufferFlush (struct cBuffer_ *buffer)
{

   /* ----- lokale Variablen ----- */
   unsigned cpsr;


   /* ----- Programmcode ----- */
   cpsr = disableIRQ ();

   buffer->len = 0;
   buffer->index = 0;

   restoreIRQ (cpsr);
}
/* ----- Ende cBufferFlush ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cBufferAdd

   Fuegt dem Buffer ein Zeichen hinzu

   ----------------------------------------------------------------------------
*/
unsigned char cBufferAdd (struct cBuffer_ *buffer, unsigned char data)
{
   /* ----- lokale Variablen ----- */
   unsigned cpsr;


   /* ----- Programmcode ----- */
   cpsr = disableIRQ ();

   if (buffer->size > buffer->len)
   {
      buffer->data[ (buffer->index + buffer->len++) % buffer->size] = data;
      restoreIRQ (cpsr);
      return 1;
   }
   restoreIRQ (cpsr);
   return 0;

}
/* ----- Ende cBufferAdd --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cBufferAddBuffer

   Fuegt dem Buffer den Inhalt eines 2. Buffers mit der Laenge len hinzu

   ----------------------------------------------------------------------------
*/
unsigned short cBufferAddBuffer (struct cBuffer_ *buffer, const void* data, unsigned short len)
{
   /* ----- lokale Variablen ----- */
   const unsigned char *su = data;
   unsigned short lenw;
   unsigned short start;
   unsigned short end;
   unsigned cpsr;


   /* ----- Programmcode ----- */
   cpsr = disableIRQ ();

   lenw = 0;

   if (buffer->size > buffer->len)
   {
      start = (buffer->index + buffer->len) % buffer->size;
      if (buffer->index)
         end = buffer->index;
      else
         end = buffer->size;

      if (end <= start)
      {
         if (buffer->size - start < len)
         {
            lenw = buffer->size - start;
            memcpy (&buffer->data[start], su, lenw);
         }
         else
         {
            memcpy (&buffer->data[start], su, len);
            buffer->len += len;
            restoreIRQ (cpsr);
            return len;
         }
         start = 0;
      }

      if (end - start < len - lenw)
      {
         memcpy (&buffer->data[start], &su [lenw], end - start);
         buffer->len = buffer->size;
         restoreIRQ (cpsr);
         return lenw + end - start;
      }
      else
      {
         memcpy (&buffer->data[start], &su [lenw] ,len-lenw);
         buffer->len += len;
         restoreIRQ (cpsr);
         return len;
      }
   }   // if (buffer->size > buffer->len)
   restoreIRQ (cpsr);
   return 0;
}   // cBufferAddBuffer
/* ----- Ende cBufferAddBuffer --------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cBufferGet

   Return one byte from the buffer an set index and length appropriately

   ----------------------------------------------------------------------------
*/
unsigned char cBufferGet (struct cBuffer_ *buffer)
{
   /* ----- lokale Variablen ----- */
   unsigned char data = 0;
   unsigned cpsr;


   /* ----- Programmcode ----- */
   cpsr = disableIRQ ();

   if (buffer->len)
   {
      data = buffer->data[buffer->index++];
      buffer->index %= buffer->size;
      buffer->len--;
   }

   restoreIRQ (cpsr);
   return data;
}
/* ----- Ende cBufferGet ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cBufferGetBuffer

   Return len bytes from the buffer an set index and length appropriately

   ----------------------------------------------------------------------------
*/
unsigned short cBufferGetBuffer (struct cBuffer_ *buffer, unsigned char* data, unsigned short len)
{
   /* ----- lokale Variablen ----- */
   unsigned cpsr;


   /* ----- Programmcode ----- */
   cpsr = disableIRQ ();

   if (buffer->len)
   {
      if (len > buffer->len)
         len = buffer->len;

      if (data)
      {
         if (buffer->index + len <= buffer->size)
         {
            memcpy (data, &buffer->data[buffer->index], len);
         }
         else
         {
            memcpy (data, &buffer->data[buffer->index], buffer->size - buffer->index);
            memcpy (&data[buffer->size - buffer->index], buffer->data, len - (buffer->size - buffer->index));
         }
      }
      buffer->index = (buffer->index + len) % buffer->size;
      buffer->len -= len;

      restoreIRQ (cpsr);
      return len;
   }
   restoreIRQ (cpsr);
   return 0;
}   // cBufferGetBuffer
/* ----- Ende cBufferGetBuffer ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cBufferLookBuffer

   Return len bytes from the buffer
   but keep the index and length of the buffer untouched

   ----------------------------------------------------------------------------
*/
unsigned short cBufferLookBuffer (struct cBuffer_ *buffer, unsigned char* data, unsigned short pos, unsigned short len)
{

   /* ----- Programmcode ----- */
   if (!buffer->len)
      return 0;

   if (pos >= buffer->len)
      return 0;

   if (len > buffer->len - pos)
      len = buffer->len - pos;

   pos += buffer->index;
   pos %= buffer->size;

   if (data)
   {
      if (pos + len <= buffer->size)
      {
         memcpy (data, &buffer->data [pos], len);
      }
      else
      {
         memcpy (data, &buffer->data [pos], buffer->size - pos);
         memcpy (&data [buffer->size - pos], buffer->data, len - (buffer->size - pos));
      }
   }

   return len;
}   // cBufferLookBuffer
/* ----- Ende cBufferLookBuffer ------------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cBufferCmp

    The return value indicates the relationship between the buffers.

   Relationship of First count Bytes of buffer and buf2
   < 0 buffer less than buf2
   0 buffer identical to buf2
   > 0 buffer greater than buf2

   ----------------------------------------------------------------------------
*/
int cBufferCmp (cBuffer_p pBuffer, const void* buf2, int count)
{
   /* ----- lokale Variablen ----- */
   const unsigned char *su2;
   int srcIdx;
   signed char res = 0;


   /* ----- Programmcode ----- */
   if (!buf2)
      return 1;

   for (srcIdx = pBuffer->index, su2 = buf2; 0 < count; ++srcIdx, ++su2, count--)
   {
      srcIdx %= pBuffer->size;
      if ((res = pBuffer->data [srcIdx] - *su2) != 0)
         break;
   }

   return res;
}   // cBufferCmp
/* ----- Ende cBufferCmp --------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: cBufferStr

   Find a substring.
   The function behaves like @see strstr

   @param pos start position for search in buffer
   @param count of by bytes to be searched

   @return the position of the first occurrence
           -1 if the buf2 has not been found in buffer
   ----------------------------------------------------------------------------
*/
int cBufferStr (cBuffer_p pBuffer, const void* buf2, unsigned short pos, unsigned short len)
{
   /* ----- lokale Variablen ----- */
   const unsigned char *su2;
   int srcIdx;
   int count;
   unsigned short buffLen = pBuffer->len;   // because len could be changed by interrupt
   signed char res = 0;


   /* ----- Programmcode ----- */
   if (!buf2 || !len)
      return -1;

   if (pos + len > buffLen)
      return -1;

   for (pos = pos; pos < buffLen; pos++)
   {
      for (srcIdx = pBuffer->index + pos, su2 = buf2, count = len;
           count > 0;
           ++srcIdx, ++su2, count--)
      {
         srcIdx %= pBuffer->size;
         if ((res = pBuffer->data [srcIdx] - *su2) != 0)
            break;
      }
      if (res == 0)
         return pos;
   }


   return -1;
}   // cBufferStr
/* ----- Ende cBufferStr --------------------------------------------------- */


/* =================================== Ende ================================ */

