/**********************************************************************
@file       drawIcon.c

@brief      Draw Black/White and Gray scale icons with or without
            Alpha channel (Transparency)

-----------------------------------------------------------------------
@author  2010 Roman Steiger
         LeoTheLoewe [at] gmx [dot] de
-----------------------------------------------------------------------
Following compile flags are usable:

FF_ICON_TRANSPARENT   0/1

-----------------------------------------------------------------------
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------
@History

1.00 20012010 RSt  Creation

@end
***********************************************************************/

/* ============================================================================
   Markus Fulde                                                      13.02.2011
   ----------------------------------------------------------------------------

   Der Sourcecode fr das WorkFrame fr die Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden
   teilweise nicht bentigte Funktionen aus Grnden der Speichergre und
   Laufzeitoptimierung (z.B. beim Download der Firmware) komplett entfernt.

   Der in dieser Version nun vorliegende Programmcode dient zur Kommunikation
   der Betty ber Funk mit dem Master-Modul des StarLight-Projekts.

   Aufgrund der groen nderungen und Ergnzungen am Code stelle ich die SW
   nicht erneut ber das Bettyhacks-Forum zur Verfgung.
   Ich erhebe keinen Anspruch des Copyrights auf jegliche SW-Teile dieses
   Projekts. Das Copyright und die Rechte am Code verbleiben in uneingeschrnker
   Forum unter Einhaltung der GNU General Public License bestehen.

   Mge jeder der den Code verwenden kann genauso viel Freude haben wie ich
   hatte!
   ============================================================================
   Historie:
   13.02.2011  Version 1  Erste vollstndige Version nach Optimierung und
                          Einbindung in Gesamtprojekt.
   ============================================================================
*/

/* =================================== Includes ============================ */


#include "global.h"
#include "lcd.h"
#include "icon.h"


/* ================== Datentypen und Variablendefinitionen ================= */

#ifndef FF_ICON_TRANSPARENT
#define FF_ICON_TRANSPARENT   0
#endif

//-----------------------------------------------------------------------------
// Internally used functions

static void _drawGray            (unsigned char x, unsigned char y,
                                  const unsigned char const *pData,
                                  unsigned char width, unsigned char height, iconInfo_t iconInfo,
                                  unsigned char m);
#if FF_ICON_TRANSPARENT
static void _drawGrayAlpha       (unsigned char x, unsigned char y,
                                  const unsigned char const *pData,
                                  unsigned char width, unsigned char height, iconInfo_t iconInfo,
                                  unsigned char m);
#endif   // #if FF_ICON_TRANSPARENT
static void _drawBlackWhite      (unsigned char x, unsigned char y,
                                  const unsigned char const *pData,
                                  unsigned char width, unsigned char height, iconInfo_t iconInfo,
                                  unsigned char c, unsigned char m);
#if FF_ICON_TRANSPARENT
static void _drawBlackWhiteAlpha (unsigned char x, unsigned char y,
                                  const unsigned char const *pData,
                                  unsigned char width, unsigned char height, iconInfo_t iconInfo,
                                  unsigned char c, unsigned char m);
#endif   // #if FF_ICON_TRANSPARENT
//-----------------------------------------------------------------------------



/* ======================= Funktionen und Programmcode ===================== */


/*
   ----------------------------------------------------------------------------
   Funktion: drawIcon
   ----------------------------------------------------------------------------
*/
void drawIcon (unsigned char x, unsigned char y, const icon_t const *pIcon, unsigned char c, unsigned char m)
{
   /* ----- Programmcode ----- */
   if (!pIcon)
      return;

   drawIconExt (x, y, pIcon->data,
      pIcon->width, pIcon->height, pIcon->info,
      c, m);
}
/* ----- Ende drawIcon ----------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: drawIconExt
   ----------------------------------------------------------------------------
*/
void drawIconExt (unsigned char x, unsigned char y,
                  const unsigned char const *pData,
                  unsigned char width, unsigned char height, iconInfo_t iconInfo,
                  unsigned char c, unsigned char m)
{
   /* ----- Programmcode ----- */
   switch (iconInfo)
   {
      case ICON_BLACK_WHITE:   /// 1 bpp
         _drawBlackWhite (x, y, pData,
            width, height, iconInfo,
            c, m);
      break;

#if FF_ICON_TRANSPARENT
      case ICON_BLACK_WHITE_TRANSPARENT:   /// 2 bpp
         _drawBlackWhiteAlpha (x, y, pData,
            width, height, iconInfo,
            c, m);
      break;
#endif   // #if FF_ICON_TRANSPARENT

      case ICON_GRAY:   /// 2 bpp
         _drawGray (x, y, pData,
            width, height, iconInfo,
            m);
      break;

#if FF_ICON_TRANSPARENT
      case ICON_GRAY_TRANSPARENT:   /// 3 bpp
         _drawGrayAlpha (x, y, pData,
            width, height, iconInfo,
            m);
      break;
#endif   // #if FF_ICON_TRANSPARENT

      default:
         if (x + width > LCD_SIZE_X)
            width = LCD_SIZE_X - x;

         if (y + height > LCD_SIZE_Y)
            height = LCD_SIZE_Y - y;

         draw_rect (x, y,
                    width, height,
                    1, LCD_COLOR_B, DRAW_XOR);

      break;
   }   // switch (iconInfo)
}
/* ----- Ende drawIconExt -------------------------------------------------- */


/*
   ----------------------------------------------------------------------------
   Funktion: _drawGray
   ----------------------------------------------------------------------------
*/
static void _drawGray (unsigned char x, unsigned char y,
                       const unsigned char const *pData,
                       unsigned char width, unsigned char height, iconInfo_t iconInfo,
                       unsigned char m)
{

   /* ----- lokale Variablen ----- */
   unsigned short ix;
   unsigned short iy;
   unsigned char  iconWidth = width;

   if (x + width > LCD_SIZE_X)
      width = LCD_SIZE_X - x;

   if (y + height > LCD_SIZE_Y)
      height = LCD_SIZE_Y - y;

   unsigned short idx = (width << 1) -1;   // 2 byte per pixel
   unsigned char s = y & 0x07;


   /* ----- Programmcode ----- */
   if (s)
   {
      for (ix = 0; ix < width; ix++)
      {
         drawbuf [1][ix] = 0;
         drawbuf [0][ix] = 0;
      }   // for (ix = 0; ix < width; ix++)

      for (iy = 0; iy < height; iy += 8)
      {
         // Need to process two icon lines for one drawbuf line
         if (iy)
         {
            for (ix = 0; ix < width; ix++)
            {
               drawbuf [1][ix] = pData [idx--] >> (8-s);
               drawbuf [0][ix] = pData [idx--] >> (8-s);
            }   // for (ix = 0; ix < width; ix++)

            // go back to end of this page
            idx += (width << 1);   // 2bpp
            // goto next page
            idx += (iconWidth << 1);   // 2bpp
         }
         for (ix = 0; ix < width; ix++)
         {
            drawbuf [1][ix] |= pData [idx--] << s;
            drawbuf [0][ix] |= pData [idx--] << s;
         }
         do_rcu (x, y+iy, width, m);

         // go back to end of this page
         idx += (width << 1);   // 2bpp
      }   // for (iy = 0; iy < height; iy += 8)

      /* Pixel in the last page: height & 0x07
         (i.e. 13 hight => 5 pixel in last page) */
      if (((height & 0x07) == 0) ||
          ((height & 0x07) > (8 - s)))
      {
         for (ix = 0; ix < width; ix++)
         {
            drawbuf [1][ix] = pData [idx--] >> (8-s);
            drawbuf [0][ix] = pData [idx--] >> (8-s);
         }   // for (ix = 0; ix < width; ix++)
         do_rcu (x, y+iy, width, m);

      }
   }   // if (s)
   else
   {
      for (iy = 0; iy < height; iy += 8)
      {
         for (ix = 0; ix < width; ix++)
         {
            drawbuf [1][ix] = pData [idx--];
            drawbuf [0][ix] = pData [idx--];
         }
         do_rcu (x, y+iy, width, m);

         // go back to end of this page
         idx += (width << 1);   // 2bpp
         // goto next page
         idx += (iconWidth << 1);   // 2bpp
      }   // for (iy = 0; iy < height; iy += 8)
   }   // else if (s)
}
/* ----- Ende _drawGray ---------------------------------------------------- */


#if FF_ICON_TRANSPARENT

/*
   ----------------------------------------------------------------------------
   Funktion: _drawGrayAlpha
   ----------------------------------------------------------------------------
*/
static void _drawGrayAlpha (unsigned char x, unsigned char y,
                            const unsigned char const *pData,
                            unsigned char width, unsigned char height, iconInfo_t iconInfo,
                            unsigned char m)
{

   /* ----- lokale Variablen ----- */
   unsigned char alphaBuf [LCD_SIZE_X];
   unsigned short ix;
   unsigned short iy;
   unsigned char  iconWidth = width;

   if (x + width > LCD_SIZE_X)
      width = LCD_SIZE_X - x;

   if (y + height > LCD_SIZE_Y)
      height = LCD_SIZE_Y - y;

   unsigned short idx = (width * 3) -1;   // 3 byte per pixel
   unsigned char s = y & 0x07;


   /* ----- Programmcode ----- */
   if (s)
   {
      for (ix = 0; ix < width; ix++)
      {
         alphaBuf   [ix] = 0;
         drawbuf [1][ix] = 0;
         drawbuf [0][ix] = 0;
      }   // for (ix = 0; ix < width; ix++)

      for (iy = 0; iy < height; iy += 8)
      {
         // Need to process two icon lines for one drawbuf line
         if (iy)
         {
            for (ix = 0; ix < width; ix++)
            {
               alphaBuf   [ix] = pData [idx--] >> (8-s);
               drawbuf [1][ix] = pData [idx--] >> (8-s);
               drawbuf [0][ix] = pData [idx--] >> (8-s);
            }   // for (ix = 0; ix < width; ix++)

            // go back to end of this page
            idx += (width * 3);   // 3bpp
            // goto next page
            idx += (iconWidth * 3);   // 3bpp
         }
         for (ix = 0; ix < width; ix++)
         {
            alphaBuf   [ix] |= pData [idx--] << s;
            drawbuf [1][ix] |= pData [idx--] << s;
            drawbuf [0][ix] |= pData [idx--] << s;
         }
         do_rcuAlpha (x, y+iy, width, m, alphaBuf);

         // go back to end of this page
         idx += (width * 3);   // 3bpp
      }   // for (iy = 0; iy < height; iy += 8)

      /* Pixel in the last page: height & 0x07
         (i.e. 13 hight => 5 pixel in last page) */
      if (((height & 0x07) == 0) ||
          ((height & 0x07) > (8 - s)))
      {
         for (ix = 0; ix < width; ix++)
         {
            alphaBuf   [ix] = pData [idx--] >> (8-s);
            drawbuf [1][ix] = pData [idx--] >> (8-s);
            drawbuf [0][ix] = pData [idx--] >> (8-s);
         }   // for (ix = 0; ix < width; ix++)
         do_rcuAlpha (x, y+iy, width, m, alphaBuf);

      }
   }
   else
   {
      for (iy = 0; iy < height; iy += 8)
      {
         for (ix = 0; ix < width; ix++)
         {
            alphaBuf   [ix] = pData [idx--];
            drawbuf [1][ix] = pData [idx--];
            drawbuf [0][ix] = pData [idx--];
         }
         do_rcuAlpha (x, y+iy, width, m, alphaBuf);

         // go back to end of this page
         idx += (width * 3);   // 3bpp
         // goto next page
         idx += (iconWidth * 3);   // 3bpp
      }   // for (iy = 0; iy < height; iy += 8)
   }   // else if (s)
}
/* ----- Ende _drawGrayAlpha ----------------------------------------------- */

#endif   // #if FF_ICON_TRANSPARENT


/*
   ----------------------------------------------------------------------------
   Funktion: _drawBlackWhite
   ----------------------------------------------------------------------------
*/
static void _drawBlackWhite (unsigned char x, unsigned char y,
                             const unsigned char const *pData,
                             unsigned char width, unsigned char height, iconInfo_t iconInfo,
                             unsigned char c, unsigned char m)
{

   /* ----- lokale Variablen ----- */
   unsigned short ix;
   unsigned short iy;
   unsigned char  iconWidth = width;

   if (x + width > LCD_SIZE_X)
      width = LCD_SIZE_X - x;

   if (y + height > LCD_SIZE_Y)
      height = LCD_SIZE_Y - y;

   unsigned short idx = width -1;   // 1 byte per pixel
   unsigned char s = y & 0x07;


   /* ----- Programmcode ----- */
   if (s)
   {
      for (ix = 0; ix < width; ix++)
      {
         drawbuf [1][ix] = 0;
         drawbuf [0][ix] = 0;
      }   // for (ix = 0; ix < width; ix++)

      for (iy = 0; iy < height; iy += 8)
      {
         // Need to process two icon lines for one drawbuf line
         if (iy)
         {
            for (ix = 0; ix < width; ix++)
            {
               if (c & 0x02)
                  drawbuf [0][ix] = pData [idx] >> (8-s);
               else
                  drawbuf [0][ix] = 0;

               if (c & 0x01)
                  drawbuf [1][ix] = pData [idx] >> (8-s);
               else
                  drawbuf [1][ix] = 0;

               idx--;
            }   // for (ix = 0; ix < width; ix++)

            // go back to end of this page
            idx += width;   // 1bpp
            // goto next page
            idx += iconWidth;   // 1bpp
         }
         for (ix = 0; ix < width; ix++)
         {
            if (c & 0x02)
               drawbuf [0][ix] |= pData [idx] << s;

            if (c & 0x01)
               drawbuf [1][ix] |= pData [idx] << s;

            idx--;
         }
         do_rcu (x, y+iy, width, m);

         // go back to end of this page
         idx += width;   // 1bpp
      }   // for (iy = 0; iy < height; iy += 8)

      /* Pixel in the last page: height & 0x07
         (i.e. 13 hight => 5 pixel in last page) */
      if (((height & 0x07) == 0) ||
         ((height & 0x07) > (8 - s)))
      {
         for (ix = 0; ix < width; ix++)
         {
            if (c & 0x02)
               drawbuf [0][ix] = pData [idx] >> (8-s);
            else
               drawbuf [0][ix] = 0;

            if (c & 0x01)
               drawbuf [1][ix] = pData [idx] >> (8-s);
            else
               drawbuf [1][ix] = 0;

            idx--;
         }   // for (ix = 0; ix < width; ix++)
         do_rcu (x, y+iy, width, m);

      }
   }
   else
   {
      for (iy = 0; iy < height; iy += 8)
      {
         for (ix = 0; ix < width; ix++)
         {
            if (c & 0x02)
               drawbuf [0][ix] = pData [idx];
            else
               drawbuf [0][ix] = 0;

            if (c & 0x01)
               drawbuf [1][ix] = pData [idx];
            else
               drawbuf [1][ix] = 0;

            idx--;
         }
         do_rcu (x, y+iy, width, m);

         // go back to end of this page
         idx += width;   // 1bpp
         // goto next page
         idx += iconWidth;   // 1bpp
      }   // for (iy = 0; iy < height; iy += 8)
   }   // else if (s)
}
/* ----- Ende _drawBlackWhite ---------------------------------------------- */


#if FF_ICON_TRANSPARENT
/*
   ----------------------------------------------------------------------------
   Funktion: _drawBlackWhiteAlpha
   ----------------------------------------------------------------------------
*/
static void _drawBlackWhiteAlpha (unsigned char x, unsigned char y,
                                  const unsigned char const *pData,
                                  unsigned char width, unsigned char height, iconInfo_t iconInfo,
                                  unsigned char c, unsigned char m)
{

   /* ----- lokale Variablen ----- */
   unsigned char alphaBuf [LCD_SIZE_X];
   unsigned short ix;
   unsigned short iy;
   unsigned char  iconWidth = width;

   if (x + width > LCD_SIZE_X)
      width = LCD_SIZE_X - x;

   if (y + height > LCD_SIZE_Y)
      height = LCD_SIZE_Y - y;

   unsigned short idx = width -1;   // 1 byte per pixel
   unsigned char s = y & 0x07;


   /* ----- Programmcode ----- */
   if (s)
   {
      for (ix = 0; ix < width; ix++)
      {
         alphaBuf   [ix] = 0;
         drawbuf [1][ix] = 0;
         drawbuf [0][ix] = 0;
      }   // for (ix = 0; ix < width; ix++)

      for (iy = 0; iy < height; iy += 8)
      {
         // Need to process two icon lines for one drawbuf line
         if (iy)
         {
            for (ix = 0; ix < width; ix++)
            {
               alphaBuf      [ix] = pData [idx--] >> (8-s);
               if (c & 0x02)
                  drawbuf [0][ix] = pData [idx] >> (8-s);
               else
                  drawbuf [0][ix] = 0;

               if (c & 0x01)
                  drawbuf [1][ix] = pData [idx] >> (8-s);
               else
                  drawbuf [1][ix] = 0;

               idx--;
            }   // for (ix = 0; ix < width; ix++)

            // go back to end of this page
            idx += width;   // 1bpp
            // goto next page
            idx += iconWidth;   // 1bpp
         }
         for (ix = 0; ix < width; ix++)
         {
            alphaBuf      [ix] = pData [idx--] << s;
            if (c & 0x02)
               drawbuf [0][ix] |= pData [idx] << s;

            if (c & 0x01)
               drawbuf [1][ix] |= pData [idx] << s;

            idx--;
         }
         do_rcuAlpha (x, y+iy, width, m, alphaBuf);

         // go back to end of this page
         idx += width;   // 1bpp
      }   // for (iy = 0; iy < height; iy += 8)

      /* Pixel in the last page: height & 0x07
         (i.e. 13 hight => 5 pixel in last page) */
      if (((height & 0x07) == 0) ||
         ((height & 0x07) > (8 - s)))
      {
         for (ix = 0; ix < width; ix++)
         {
            alphaBuf      [ix] = pData [idx--] >> (8-s);
            if (c & 0x02)
               drawbuf [0][ix] = pData [idx] >> (8-s);
            else
               drawbuf [0][ix] = 0;

            if (c & 0x01)
               drawbuf [1][ix] = pData [idx] >> (8-s);
            else
               drawbuf [1][ix] = 0;

            idx--;
         }   // for (ix = 0; ix < width; ix++)

         do_rcuAlpha (x, y+iy, width, m, alphaBuf);
      }
   }
   else
   {
      for (iy = 0; iy < height; iy += 8)
      {
         for (ix = 0; ix < width; ix++)
         {
            alphaBuf      [ix] = pData [idx--];
            if (c & 0x02)
               drawbuf [0][ix] = pData [idx];
            else
               drawbuf [0][ix] = 0;

            if (c & 0x01)
               drawbuf [1][ix] = pData [idx];
            else
               drawbuf [1][ix] = 0;

            idx--;
         }
         do_rcuAlpha (x, y+iy, width, m, alphaBuf);

         // go back to end of this page
         idx += width;   // 1bpp
         // goto next page
         idx += iconWidth;   // 1bpp
      }   // for (iy = 0; iy < height; iy += 8)
   }   // else if (s)
}
/* ----- Ende _drawBlackWhiteAlpha ----------------------------------------- */

#endif   // #if FF_ICON_TRANSPARENT


/* =================================== Ende ================================ */



