/* ============================================================================
   Markus Fulde                                                      06.01.2011
   ----------------------------------------------------------------------------

   Der Sourcecode der Betty stammt aus dem Forum
                        http://www.bettyhacks.com
   und wurde speziell fr das Projekt StarLight angepasst. Dabei wurden nicht
   bentigte Funktionen aus Grnden der Speichergre und Laufzeitoptimierung
   komplett entfernt.

   Verschiedene Funktionen wurden erweitert oder umgeschrieben.

   Der in dieser Version nun vorliegende Programmcode dient zur Steuerung
   des Beleuctungsanlage mit Hilfe der Betty ber Funkverbindung mit dem
   Master-Modul des StarLight-Projekts.
   ============================================================================
   Historie:
   03.01.2010  Version 1  Erste vollstndige Version nach Optimierung

   ============================================================================
*/

/* ****************************************************************************
@file       Rfstarlight.c

@brief      Interface for communication with SCART adapter via RF

            Modul bildet das Interface zwischen Betty und dem StarLight
            Master an welchem der SCART Adapter angebunden ist.

-------------------------------------------------------------------------------
@author  2010 Markus Fulde
-------------------------------------------------------------------------------
Following compile flags are usable:


-------------------------------------------------------------------------------
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
@History

1.00 17092010   MaFu    Ersterstellung

@end
**************************************************************************** */

/* =================================== Includes ============================ */

#include "global.h"
#include <string.h>
#include "buffer.h"
#include "timerfuncs.h"
#include "rf.h"
#include "rfstarlight.h"
#include "starlight.h"
#include "menu.h"
#include "controls.h"
#include "lcd.h"
#include "fonty.h"
#include "keyboard.h"
#include "backlight.h"
#include "serial.h"


extern int atoi(const char *s);   // due to problems while include ctype.h

/* ================== Datentypen und Variablendefinitionen ================= */

#define RFSTARLIGHT_INIT_DEBUG   0  // Debug fr alles was mit INIT und OPEN zu tun hat
#define RFSTARLIGHT_RX_DEBUG     0  // Debug fr alles was mit Empfang von Daten zu tun hat
#define RFSTARLIGHT_TX_DEBUG     0  // Debug fr alles was mit Senden von Daten zu tun hat
#define STARLIGHT_DEBUG          0  // Debug fr allgemeine StarLight-Funktionen
#define CMDSTARLIGHT_DEBUG       0  // Debug something on RS232 - alle Funktionen fr Mastercommandierung

#define RF_BINARY_RFRX_STATE          RFwor
#define RF_BINARY_HANDSHAKE_TIMEOUT   TIME_MSEC2TICKS (2000)
#define RF_BINARY_MAX_RETRAIL         3
#define RF_BINARY_DATA_TIMEOUT        TIME_MSEC2TICKS (3000)


// Datendefinition der gesamten RF-Datenstrukturen
RfStarLightData_t   rfStarLightData;

// Datendefiniton fr StarLight-Daten, definiert in starlight.h und deklariert in starlight.c
extern starlightData_t starlightData;


/* ======================= Funktionen und Programmcode ===================== */


/* ----------------------------------------------------------------------------
Funktion: RfStarLightInit

Funktion initialisiert zu Beginn bei Systemstart die Buffer zum Senden und
Empfangen von Daten via RF.

bergabeparameter:      keiner

Rckgabeparameter:      TRUE = Funktion war erfolgreich
---------------------------------------------------------------------------- */
BOOL RfStarLightInit ( void )
{
   /* ----- lokale Variablen ----- */
   unsigned char index;

   #if RFSTARLIGHT_INIT_DEBUG
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */

   #if RFSTARLIGHT_INIT_DEBUG
      serial_puts("Enter RfStarLightInit\n");
   #endif

   // RF-Verbindung ffnen, Verbindung bleibt offen
   RfStarLightOpen ( );

   // Buffer initialisieren
   for ( index = 0 ; index < ( RF_TX_MESSAGE_LENGTH ); index++ )
   {
      rfStarLightData.TxData.TxBuffer[ index ] = 0x00;
   }
   for ( index = 0 ; index < ( RF_RX_MESSAGE_LENGTH ); index++ )
   {
      rfStarLightData.RxData.RxBuffer[ index ] = 0x00;
   }

   // Zunchst erst mal Daten vorinitialisieren
   rfStarLightData.TxData.TxData.framelength    = 3;
   rfStarLightData.TxData.TxData.kommando       = 0x00;
   rfStarLightData.TxData.TxData.parameter1     = 0xFF;
   rfStarLightData.TxData.TxData.parameter2     = 0xFF;
   rfStarLightData.TxData.TxData.checksumme_lb  = 0x00;
   rfStarLightData.TxData.TxData.checksumme_hb  = 0x00;

   // Timer initialisieren
   rfStarLightData.timer = 0;

   // Flags initialisieren
   rfStarLightData.flags = RF_BINARY_FLAG_NONE;

   // Ggf. Traces ausgeben
   #if RFSTARLIGHT_INIT_DEBUG
      serial_puts("Dateninitialisierung:\n");
      sprintf( trace_string, "RX Framelaenge : %x\n", rfStarLightData.RxData.RxData.framelength );    serial_puts( trace_string );
      sprintf( trace_string, "RX Kommando    : %x\n", rfStarLightData.RxData.RxData.kommando );       serial_puts( trace_string );
      sprintf( trace_string, "RX Dimmer      : %x\n", rfStarLightData.RxData.RxData.dimmer );         serial_puts( trace_string );
      sprintf( trace_string, "RX Dimmwert    : %x\n", rfStarLightData.RxData.RxData.dimmwert );       serial_puts( trace_string );
      sprintf( trace_string, "RX Temperatur  : %x\n", rfStarLightData.RxData.RxData.temperatur );     serial_puts( trace_string );
      sprintf( trace_string, "RX Sound       : %x\n", rfStarLightData.RxData.RxData.sound );          serial_puts( trace_string );
      sprintf( trace_string, "RX ChecksummeLB: %x\n", rfStarLightData.RxData.RxData.checksumme_lb );  serial_puts( trace_string );
      sprintf( trace_string, "RX ChecksummeHB: %x\n", rfStarLightData.RxData.RxData.checksumme_hb );  serial_puts( trace_string );
      sprintf( trace_string, "TX Framelaenge : %x\n", rfStarLightData.TxData.TxData.framelength );    serial_puts( trace_string );
      sprintf( trace_string, "TX Kommando    : %x\n", rfStarLightData.TxData.TxData.kommando );       serial_puts( trace_string );
      sprintf( trace_string, "TX Parameter 1 : %x\n", rfStarLightData.TxData.TxData.parameter1 );     serial_puts( trace_string );
      sprintf( trace_string, "TX Parameter 2 : %x\n", rfStarLightData.TxData.TxData.parameter2 );     serial_puts( trace_string );
      sprintf( trace_string, "TX ChecksummeLB: %x\n", rfStarLightData.TxData.TxData.checksumme_lb );  serial_puts( trace_string );
      sprintf( trace_string, "TX ChecksummeHB: %x\n", rfStarLightData.TxData.TxData.checksumme_hb );  serial_puts( trace_string );
   #endif

   // Rckgabewert setzen
   return true;

}
/* ----- Ende RfStarLightInit ---------------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: RfStarLightOpen

Funktion erffnet eine Funkverbindung

bergabeparameter:      keiner

Rckgabeparameter:      TRUE  = Open erfolgreich
                        FALSE = Open fehlgeschlagen
---------------------------------------------------------------------------- */
BOOL RfStarLightOpen ( void )
{
   /* ----- lokale Variablen ----- */
   #if RFSTARLIGHT_INIT_DEBUG
      char trace_string[255];
   #endif

   BOOL result_value = true;


   /* ----- Programmcode ----- */

   #if RFSTARLIGHT_INIT_DEBUG
      serial_puts("pTxEndpoint anlegen\n");
   #endif
   rfStarLightData.pTxEndpoint = openEP (rfStarLightData.TxData.TxBuffer, RF_TX_MESSAGE_LENGTH, RF_PACKET_BINARY);
   rfStarLightData.pTxEndpoint->bufferlen = RF_TX_MESSAGE_LENGTH;
   rfStarLightData.pTxEndpoint->flags  |= EPenabled | EPoutput | EPtypebuffer;

   #if RFSTARLIGHT_INIT_DEBUG
      serial_puts("pRxEndpoint anlegen\n");
   #endif
   rfStarLightData.pRxEndpoint = openEP (rfStarLightData.RxData.RxBuffer, RF_RX_MESSAGE_LENGTH, RF_PACKET_BINARY);
   rfStarLightData.pRxEndpoint->flags |= EPenabled | EPinput | EPtypebuffer;

   // Prfen ob Endpoints vorhanden sind

   // Transmit
   if (!rfStarLightData.pTxEndpoint)
   {
      // NEIN
      #if RFSTARLIGHT_INIT_DEBUG
         serial_puts( "rfStarLightData.pTxEndpoint SCHLECHT\n" );
      #endif
      result_value = false;
   }
   #if RFSTARLIGHT_INIT_DEBUG
      else
      {
         // JA
         serial_puts( "rfStarLightData.pTxEndpoint GUT\n" );
         sprintf ( trace_string , "Adresse von pTxEndpoint: %x\n", (void *)rfStarLightData.pTxEndpoint );
         serial_puts( trace_string );
      }
   #endif

   // Receive
   if (!rfStarLightData.pRxEndpoint)
   {
      // NEIN
      #if RFSTARLIGHT_INIT_DEBUG
         serial_puts( "rfStarLightData.pRxEndpoint SCHLECHT\n" );
      #endif
      result_value = false;
   }
   #if RFSTARLIGHT_INIT_DEBUG
      else
      {
         // JA
         serial_puts( "rfStarLightData.pRxEndpoint GUT\n" );
         sprintf ( trace_string , "Adresse von pRxEndpoint: %x\n", (void *)rfStarLightData.pRxEndpoint );
         serial_puts( trace_string );
      }
   #endif

   // Rckgabewert setzen
   return result_value;

}
/* ----- Ende RfStarLightOpen ---------------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: RfStarLightSend

Funktion sendet den Sendebuffer via RF an den Master und empfngt die ent-
sprechende Antwort in dem Empfangbuffer.

bergabeparameter:      TRUE  = Es wird auf eine Antwort gewartet
                        FALSE = Keine Antwort erforderlich

Rckgabeparameter:      TRUE  = Es wurden Daten emfpangen oder ein Timeout ist
                                aufgetreten.
                        FALSE = Funktion ohne Erfolg gelaufen

Bemerkung: Durch den Aufruf RfStarLightGetState kann abgefragt werden, on
           Daten empfangen wurden oder ein Timeout aufgetreten ist.
---------------------------------------------------------------------------- */
BOOL RfStarLightSend ( void )
{
   /* ----- lokale Variablen ----- */
   BOOL           receive_ready     = false;
   unsigned int   temp_int;
   unsigned char  retrailCounter    = 0;
   unsigned       cpsr;

   #if RFSTARLIGHT_TX_DEBUG
      char           trace_string_tx[255];
      unsigned char  index_tx;
   #endif

   #if RFSTARLIGHT_RX_DEBUG
      char           trace_string_rx[255];
      unsigned char  index_rx;
   #endif


   /* ----- Programmcode ----- */

   #if RFSTARLIGHT__TX_DEBUG
      serial_puts("Enter RfStarLightSend\n");
   #endif

   // Prfen ob fr das Senden und Empfangen von Daten eine Verbindung existiert
   if ( rfStarLightData.pTxEndpoint && rfStarLightData.pRxEndpoint )
   {
      // JA

      rfStarLightData.flags = RF_BINARY_FLAG_NONE;

      // disable interrupts while manipulating buffer
      cpsr = disableIRQ ();
         // Flag fr neue Daten zurcksetzen
         rfStarLightData.pRxEndpoint->flags &= ~EPnewdata;
      restoreIRQ (cpsr);

      // ChecSumme berechnen und eintragen
      temp_int = CRC16( rfStarLightData.TxData.TxBuffer, 4 );

      // Ablage der CRC im Speicher zuerst LB, dann HB
      rfStarLightData.TxData.TxData.checksumme_lb = temp_int & 0x000000FF;
      rfStarLightData.TxData.TxData.checksumme_hb = ( temp_int >> 8 ) & 0x000000FF;

      #if RFSTARLIGHT_TX_DEBUG
         serial_puts("Es sollen gesendet werden: ");
         for ( index_tx = 0; index_tx < RF_TX_MESSAGE_LENGTH; index_tx++ )
         {
            sprintf(trace_string_tx, " %x ", rfStarLightData.TxData.TxBuffer[ index_tx ]); serial_puts( trace_string_tx );
         }
         serial_puts( "\n" );
         sprintf(trace_string_tx, "pTxEndpoint->bufferlen %x \n", rfStarLightData.pTxEndpoint->bufferlen );    serial_puts( trace_string_tx );
         sprintf(trace_string_tx, "pTxEndpoint->buffersize %x \n", rfStarLightData.pTxEndpoint->buffersize );  serial_puts( trace_string_tx );
         sprintf(trace_string_tx, "pTxEndpoint->type %x \n", rfStarLightData.pTxEndpoint->type );              serial_puts( trace_string_tx );
      #endif

      // disable interrupts while manipulating buffer
      cpsr = disableIRQ ();
         // Flag neuer Daten fr RX setzen
         rfStarLightData.pTxEndpoint->flags |= EPnewdata;
      restoreIRQ (cpsr);

      // Daten senden
      RF_changestate ( RFtx ); // transmit
      //RF_changestate ( RF_BINARY_RFRX_STATE ); // RFwor = Wake On Radio
      RF_changestate ( RFwor ); // RFwor = Wake On Radio

      // Empfange die Antwort
      do
      {
         #if RFSTARLIGHT_RX_DEBUG
            sprintf(trace_string_rx, "\npretrailCounter : %d \n", retrailCounter ); serial_puts( trace_string_rx );
            sprintf(trace_string_rx, "Bufferlnge : %d \n", rfStarLightData.pRxEndpoint->bufferlen ); serial_puts( trace_string_rx );
         #endif

         // Timer zur berwachung der Master-Antwort aufziehen
         addTimeout (&rfStarLightData.timer, RF_BINARY_HANDSHAKE_TIMEOUT);

         while ( rfStarLightData.timer && !receive_ready )
         {

            // Prfen ob berhaupt neue Daten empfangen wurden
            if ( rfStarLightData.pRxEndpoint->flags & EPnewdata )
            {
               // JA

               // Prfung ob ausreichend Bytes empfangen wurden
               if ( rfStarLightData.pRxEndpoint->bufferlen >= RF_RX_MESSAGE_LENGTH )
               {
                  //Ja

                  #if RFSTARLIGHT_RX_DEBUG
                     sprintf(trace_string_rx, "\npRxEndpoint->bufferlen: %x \n", rfStarLightData.pRxEndpoint->bufferlen ); serial_puts( trace_string_rx );

                     serial_puts( "\nRfStarLightSend: Ausgabe empfangener Bytes pRxEndpoint:\n" );
                     for ( index_rx = 0 ; index_rx < RF_RX_MESSAGE_LENGTH ; index_rx++ )
                     {
                        sprintf ( trace_string_rx , "%x ", rfStarLightData.pRxEndpoint->buffer[ index_rx ] ); serial_puts( trace_string_rx );
                     }
                     serial_puts( "\n" );

                     serial_puts( "\nRfStarLightSend: Ausgabe empfangener Bytes rfStarLightData:\n" );
                     for ( index_rx = 0 ; index_rx < RF_RX_MESSAGE_LENGTH ; index_rx++ )
                     {
                        sprintf ( trace_string_rx , "%x ", rfStarLightData.RxData.RxBuffer[ index_rx ] ); serial_puts( trace_string_rx );
                     }
                     serial_puts( "\n" );
                  #endif

                  // Receive-Status auf TRUE setzen
                  receive_ready = true;
               } // Ausreichend Bytes empfangen
            } // Noch keine neuen Daten empfangen
         } // Timeout oder Bytes empfangen

         if ( !receive_ready )
         {
            // Repeat the last message
            if ( ++retrailCounter <= RF_BINARY_MAX_RETRAIL)
            {
               cpsr = disableIRQ ();
                  // Flag neuer Daten fr RX setzen
                  rfStarLightData.pTxEndpoint->flags |= EPnewdata;
               restoreIRQ (cpsr);
               RF_changestate ( RFtx );
               // RF_changestate ( RF_BINARY_RFRX_STATE );
               RF_changestate ( RFrx );
            }
         }   // if (!isHandshake)

      } while ( !receive_ready &&
                retrailCounter <= RF_BINARY_MAX_RETRAIL );

      // Timeout aus Scheduler-Queue lschen
      removeTimeout (&rfStarLightData.timer);

   } // Existiert P2P Verbindung

   rfStarLightData.flags = RF_BINARY_FLAG_NONE;
   // RF_changestate (RFwor);
   RF_changestate (RFwor);

   return receive_ready;

}
//----- Ende RfStarLightSend --------------------------------------------------


//   /* ----------------------------------------------------------------------------
//   Funktion: RfStarLightDone
//
//   Diese Funktion mu aufgerufen werden, wenn der Rckgabewert von RfStarLightSend
//   TRUE zurckliefert.
//   Wenn RfStarLightSend TRUE zurck liefert so wurden entweder Daten empfangen
//   oder ein Timeout ist abgelaufen.
//
//
//   bergabeparameter:      TRUE  = Es wird auf eine Antwort gewartet
//                           FALSE = Keine Antwort erforderlich
//
//   Rckgabeparameter:      TRUE  = Es wurden Daten emfpangen oder ein Timeout ist
//                                   aufgetreten.
//                           FALSE = Funktion ohne Erfolg gelaufen
//
//   Bemerkung: Durch den Aufruf RfStarLightGetState kann abgefragt werden, on
//              Daten empfangen wurden oder ein Timeout aufgetreten ist.
//   ---------------------------------------------------------------------------- */
//   void RfStarLightDone ( void )
//   {
//
//   /*
//      Bei RfStarLightDone wird die Endpunkt-Datenstruktur nicht geschlossen da dauerhaft offen
//
//      // Prfen ob es einen RX-Endpunkt gibt
//      if (rfStarLightData.pRxEndpoint)
//      {
//         // Ja
//
//         // Endpunkt schlieen und Pointer wieder auf NULL setzen
//         closeEP (rfStarLightData.pRxEndpoint);
//         rfStarLightData.pRxEndpoint = NULL;
//      }
//
//      // Prfen ob es einen TX-Endpunkt gibt
//      if (rfStarLightData.pTxEndpoint)
//      {
//         // Ja
//
//         // Endpunkt schlie0en und Pointer wieder auf NULL setzen
//         closeEP (rfStarLightData.pTxEndpoint);
//         rfStarLightData.pTxEndpoint = NULL;
//      }
//
//
//      rfStarLightData.flags = RF_BINARY_FLAG_NONE;
//      removeTimeout (&rfStarLightData.timer);
//
//      RF_changestate (RFwor);
//
//   */
//
//      //sysInfo &= ~SYS_NOPDOWN;
//   }
//   //----- Ende RfStarLightDone --------------------------------------------------


//   /* ----------------------------------------------------------------------------
//   Funktion: RfStarLightGetState
//
//   Diese Funktion gibt den aktuellen Status der internen Statemachine zurck
//
//   typedef enum RfBinaryStatus_e
//   {
//      RF_BINARY_IDLE,
//      RF_BINARY_WAITING,
//      RF_BINARY_TIMEOUT,
//      RF_BINARY_ERROR,
//      RF_BINARY_DATA_RECEIVED,
//      RF_BINARY_MAX
//   } RfBinaryStatus_t;
//
//   #define RF_BINARY_FLAG_NONE                 0
//   #define RF_BINARY_FLAG_DATA_EXPECTED   _BV (0)
//   #define RF_BINARY_FLAG_TIMEOUT         _BV (1)
//
//   bergabeparameter:      keiner
//
//   Rckgabeparameter:      siehe Beschreibung der Stati
//
//   Bemerkung: Durch den Aufruf RfStarLightGetState kann abgefragt werden, ob
//              Daten empfangen wurden oder ein Timeout aufgetreten ist.
//   ---------------------------------------------------------------------------- */
//   RfBinaryStatus_t RfStarLightGetState ( void )
//   {
//      // Zustand: IDLE
//      if ( !(rfStarLightData.flags & RF_BINARY_FLAG_DATA_EXPECTED) )
//      {
//         return RF_BINARY_IDLE;
//      }
//
//      // Zustand: Neue Daten empfangen
//      if (rfStarLightData.pRxEndpoint->flags & EPnewdata)
//      {
//         return RF_BINARY_DATA_RECEIVED;
//      }
//
//      // Zustand: Timer
//      if (!rfStarLightData.timer)
//      {
//         // Prfen on Timer abgelaufen
//         if (!(rfStarLightData.flags & RF_BINARY_FLAG_TIMEOUT))
//         {
//            // JA -> TIMEOUT
//            rfStarLightData.flags |= RF_BINARY_FLAG_TIMEOUT;
//            return RF_BINARY_TIMEOUT;
//         }
//         // NEIN -> IDLE
//         return RF_BINARY_IDLE;
//      }
//
//      // Zustand: WAITING
//      return RF_BINARY_WAITING;
//   }
//   //----- Ende RfStarLightGetState ----------------------------------------------


/* ----------------------------------------------------------------------------
Funktion: RfStarLightPurgeBuffer

Funktion leert den Empfangsbuffer und bereitet Datenstrukt soweit notwendig auf
Empfang neuer Daten vor.

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void RfStarLightPurgeBuffer ( void )
{
   /* ----- lokale Variablen ----- */
   unsigned       cpsr;


   /* ----- Programmcode ----- */

   // Bufferlnge zurcksetzen
   rfStarLightData.pRxEndpoint->bufferlen = 0;

   // disable interrupts while manipulating buffer
   cpsr = disableIRQ ();
      rfStarLightData.pRxEndpoint->flags &= ~EPnewdata;
   restoreIRQ (cpsr);

}
/* ---- Ende RfStarLightPurgeBuffer ---------------------------------------- */


/* ----------------------------------------------------------------------------
Funktion: StarLightSend

Funktion sendet den Sendebuffer via RF an den Master und empfngt die ent-
sprechende Antwort in dem Empfangbuffer.

bergabeparameter:      keiner

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL StarLightSend ( unsigned char popup_flag )
{
   /* ----- lokale Variablen ----- */
   BOOL           return_value   = true;
   unsigned int   temp_uint      = 0;
   unsigned int   checksumme     = 0;

   #if STARLIGHT_DEBUG
      unsigned char index;
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */

   #if STARLIGHT_DEBUG
      serial_puts("\nEnter StarLightSend\n");
   #endif

   // Prfen ob PopUp angezeigt werden soll
   if ( popup_flag == true )
   {
      // kleiner Font setzen
      set_font (SMOOTHFONT);

      // PopUp auf Display zeigen
      msgbox (30, BC_nobuttons, "\n    Sende...    \n");
   }

   // starlightData.isTimerRunning = false;

   // Ergebnis der RF Kommunikation prfen ob Fehler aufgetaucht
   // Fehler kann hier prinzipiell nur durch Timeout entstehen
   if ( !RfStarLightSend ( ) )
   {

      msgbox (30, BC_OKOnly | BC_DefaultButton1, "StarLightSend:\nKeine Funkverbindung!\nMaster antwortet nicht\n");

/*
      // Start timer for automatic retrial
      starlightData.isTimerRunning = true;
      addTimeout ( &( starlightData.timer ), STARLIGHT_DATA_TIMEOUT);
*/
      return_value = false;
   }

   // Soooo, nun empfangene Daten ber Checksumme prfen

   checksumme = ( rfStarLightData.RxData.RxData.checksumme_hb << 8 ) + rfStarLightData.RxData.RxData.checksumme_lb;

   #if STARLIGHT_DEBUG
      sprintf ( trace_string , "CRC16: %x\n", checksumme ); serial_puts( trace_string );
   #endif

   temp_uint = CRC16( rfStarLightData.RxData.RxBuffer, 6 );

   #if STARLIGHT_DEBUG
      serial_puts( "\nStarLightSend: Ausgabe empfangener Bytes:\n" );
      for ( index = 0 ; index < RF_RX_MESSAGE_LENGTH ; index++ )
      {
         sprintf ( trace_string , "%x ", rfStarLightData.RxData.RxBuffer[ index ] ); serial_puts( trace_string );
      }
      serial_puts( "\n" );
   #endif

   #if STARLIGHT_DEBUG
      sprintf ( trace_string , "Soll: %x    Ist: %x\n\n", checksumme, temp_uint ); serial_puts( trace_string );
      serial_puts( "Empfangene Daten:\n" );
      sprintf ( trace_string , "framelength  : %x\n", rfStarLightData.RxData.RxData.framelength );    serial_puts( trace_string );
      sprintf ( trace_string , "kommando     : %x\n", rfStarLightData.RxData.RxData.kommando );       serial_puts( trace_string );
      sprintf ( trace_string , "dimmer       : %x\n", rfStarLightData.RxData.RxData.dimmer );         serial_puts( trace_string );
      sprintf ( trace_string , "dimmwert     : %x\n", rfStarLightData.RxData.RxData.dimmwert );       serial_puts( trace_string );
      sprintf ( trace_string , "temperatur   : %x\n", rfStarLightData.RxData.RxData.temperatur );     serial_puts( trace_string );
      sprintf ( trace_string , "sound        : %x\n", rfStarLightData.RxData.RxData.sound );          serial_puts( trace_string );
      sprintf ( trace_string , "checksummeHB : %x\n", rfStarLightData.RxData.RxData.checksumme_hb );  serial_puts( trace_string );
      sprintf ( trace_string , "checksummeLB : %x\n", rfStarLightData.RxData.RxData.checksumme_lb );  serial_puts( trace_string );
   #endif

   if ( checksumme != temp_uint )
   {
      #if STARLIGHT_DEBUG
         serial_puts( "StarLightSend_ Checksummenfehler\n" );
      #endif

      msgbox (30, BC_OKOnly | BC_DefaultButton1, "StarLightSend:\nKommunikationsfehler!\nFehler in Checksumme\n");
      return_value = false;
   }

   // Rckgabewert setzen
   return return_value;

}
//----- StarLightSend ---------------------------------------------------------


/* ----------------------------------------------------------------------------
Funktion: RfStarLightStatusabfrage

Funktion dient zur Statusabfrage der Betriebsstati eines Dimmers.
Rckgabewerte sind:  [1] Kommando
                     [2] Dimmer
                     [3] Dimmwert
                     [4] Temperatur
                     [5] Sound

bergabeparameter:      dimmer = Dimmer [1..8]

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightStatusabfrage ( unsigned char dimmer )
{
   // Zur Info:
   // Ist die Lichtanlage ber Fernsteuerung ausgeschaltet worden, ist der
   // Hauptschalter aber noch an so reagiert die Steuerung auf Funkbefehle.
   // Im Falle der Statusanfrage werden zwei Zustnde unterschieden:
   // Fall 1: Steuerung AUS = Auf die Anfrage wird mit ACK 0xFF geantwortet
   // Fall 2: Steuerung EIN = Es wird der komplette Status des Dimmers
   //                         zurckgegeben.

   /* ----- lokale Variablen ----- */
   unsigned char return_value = true;

   #if CMSSTARLIGHT_DEBUG
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */

   #if CMDSTARLIGHT_DEBUG
      serial_puts("Enter RfStarLightStatusabfrage\n");
   #endif

   // bertragungsdaten in SendeFrame setzen
   rfStarLightData.TxData.TxData.kommando   = CMD_BETTY_MASTER_STATUS;
   rfStarLightData.TxData.TxData.parameter1 = dimmer;
   rfStarLightData.TxData.TxData.parameter2 = BETTY_MASTER_UNDEFINED;

   // Daten senden
   return_value = StarLightSend ( NOT_SHOW_SEND_POPUP );

   // Rckgabewert setzen
   return return_value;

}
//----- Ende RfStarLightStatusabfrage -----------------------------------------


/* ----------------------------------------------------------------------------
Funktion: RfStarLightLichtanlageSchalten

Funktion ist Toggelfunktion und dient zum einschalten und ausschalten der
Lichtanlage.
Ist die Lichtanlage ausgeschaltet wird prinzipiell immer ACK mit ACK = 0
emfangen.

bergabeparameter:      keiner

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightLichtanlageSchalten ( void )
{
   /* ----- lokale Variablen ----- */
   unsigned char return_value = true;

   #if CMSSTARLIGHT_DEBUG
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */

   #if CMDSTARLIGHT_DEBUG
      serial_puts("Enter RfStarLightLichtanlageSchalten\n");
   #endif

   // bertragungsdaten in SendeFrame setzen
   rfStarLightData.TxData.TxData.kommando   = CMD_BETTY_MASTER_ANLAGE;
   rfStarLightData.TxData.TxData.parameter1 = BETTY_MASTER_UNDEFINED;
   rfStarLightData.TxData.TxData.parameter2 = BETTY_MASTER_UNDEFINED;

   // Daten senden
   return_value = StarLightSend ( NOT_SHOW_SEND_POPUP );

   // Rckgabewert setzen
   return return_value;

}
//----- Ende RfStarLightLichtanlageSchalten -----------------------------------


/* ----------------------------------------------------------------------------
Funktion: RfStarLightDimmerSteuern

Mit dieser Funktion knnen die einzelnen Dimmer der Slaves ber den Master
angesteuert werden.

bergabeparameter:      dimmer   = Auswahl Dimmer [1..8]
                        dimmwert = Dimmwert [1..31]

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightDimmerSteuern ( unsigned char dimmer , unsigned char dimmwert )
{
   /* ----- lokale Variablen ----- */
   unsigned char return_value = true;

   #if CMSSTARLIGHT_DEBUG
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */

   #if CMDSTARLIGHT_DEBUG
      serial_puts("Enter RfStarLightDimmerSteuern\n");
   #endif

   // bertragungsdaten in SendeFrame setzen
   rfStarLightData.TxData.TxData.kommando   = CMD_BETTY_MASTER_DIMWERT;
   rfStarLightData.TxData.TxData.parameter1 = dimmer;
   rfStarLightData.TxData.TxData.parameter2 = dimmwert;

   // Daten senden
   return_value = StarLightSend ( NOT_SHOW_SEND_POPUP );

   // Rckgabewert setzen
   return return_value;

}
//----- Ende RfStarLightDimmerSteuern -----------------------------------------


/* ----------------------------------------------------------------------------
Funktion: RfStarLightSoundSchalten

Funktion sendet die Sound-Einstellung zum Master und damit zu den Slaves.
ber diese Funktion wird eingestellt, ob Master und Slaves mit akustischem
Feedback bei Auswahl von Funktionen antworten.

bergabeparameter:      sound = Sound Ein/Aus [True/False]

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightSoundSchalten ( unsigned char sound )
{

   // Zur Info:
   // Ist die Lichtanlage ber Fernsteuerung ausgeschaltet worden, ist der
   // Hauptschalter aber noch an so reagiert die Steuerung auf Funkbefehle.
   // Im Falle von RfStarLightSoundSchalten wird immer ein ACK geliefert

   /* ----- lokale Variablen ----- */
   unsigned char return_value = true;

   #if CMSSTARLIGHT_DEBUG
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */

   #if CMDSTARLIGHT_DEBUG
      serial_puts("Enter RfStarLightSoundSchalten\n");
   #endif

   // bertragungsdaten in SendeFrame setzen
   rfStarLightData.TxData.TxData.kommando   = CMD_BETTY_MASTER_SOUND;
   rfStarLightData.TxData.TxData.parameter1 = sound;
   rfStarLightData.TxData.TxData.parameter2 = BETTY_MASTER_UNDEFINED;

   // Daten senden
   return_value = StarLightSend ( NOT_SHOW_SEND_POPUP );

   // Datenstrukturen zurcksetzen
   RfStarLightPurgeBuffer( );

   // Rckgabewert setzen
   return return_value;

}
//----- Ende RfStarLightSoundSchalten -----------------------------------------


/* ----------------------------------------------------------------------------
Funktion: RfStarLightGesamtlicht

Mit dieser Funktion kann das gesamte Licht aller Dimmer auf einmal
geschaltet werden.

bergabeparameter:      lichtstatus = Licht Ein/Aus [True/False]

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightGesamtlicht ( unsigned char lichtstatus )
{
   /* ----- lokale Variablen ----- */
   unsigned char return_value = true;

   #if CMSSTARLIGHT_DEBUG
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */

   #if CMDSTARLIGHT_DEBUG
      serial_puts("Enter RfStarLightGesamtlicht\n");
   #endif

// bertragungsdaten in SendeFrame setzen
   rfStarLightData.TxData.TxData.kommando   = CMD_BETTY_MASTER_LAMPEN;
   rfStarLightData.TxData.TxData.parameter1 = lichtstatus;
   rfStarLightData.TxData.TxData.parameter2 = BETTY_MASTER_UNDEFINED;

   // Daten senden
   return_value = StarLightSend ( NOT_SHOW_SEND_POPUP );

   // Rckgabewert setzen
   return return_value;

}
//----- Ende RfStarLightGesamtlicht -------------------------------------------


/* ----------------------------------------------------------------------------
Funktion: RfStarLightDimmerSchalten

Mit dieser Funktion kann ein einzelner Dimmer aus und eingeschaltet werden.

bergabeparameter:      dimmer     = Dimmer [1..8]
                        dimmstatus = Dimmer Ein/Aus [True/False]

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightDimmerSchalten ( unsigned char dimmer , unsigned char dimmstatus )
{
   /* ----- lokale Variablen ----- */
   unsigned char return_value = true;

   #if CMSSTARLIGHT_DEBUG
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */

   #if CMDSTARLIGHT_DEBUG
      serial_puts("Enter RfStarLightDimmerSchalten\n");
   #endif

   // bertragungsdaten in SendeFrame setzen
   rfStarLightData.TxData.TxData.kommando   = CMD_BETTY_MASTER_DIMMER;
   rfStarLightData.TxData.TxData.parameter1 = dimmer;
   rfStarLightData.TxData.TxData.parameter2 = dimmstatus;

   // Daten senden
   return_value = StarLightSend ( NOT_SHOW_SEND_POPUP );

   // Rckgabewert setzen
   return return_value;

}
//----- Ende RfStarLightDimmerSchalten ----------------------------------------


/* ----------------------------------------------------------------------------
Funktion: RfStarLightDemomodus

Funktion lst im Master den Demomodus aus.

bergabeparameter:      keiner

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightDemomodus ( void )
{

   // Zur Info:
   // Ist die Lichtanlage ber Fernsteuerung ausgeschaltet worden, ist der
   // Hauptschalter aber noch an so reagiert die Steuerung auf Funkbefehle.
   // Im Falle von RfStarLightSoundSchalten wird immer ein ACK geliefert

   /* ----- lokale Variablen ----- */
   unsigned char return_value = true;

   #if CMSSTARLIGHT_DEBUG
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */

   #if CMDSTARLIGHT_DEBUG
      serial_puts("Enter RfStarLightDemomodus\n");
   #endif

   // bertragungsdaten in SendeFrame setzen
   rfStarLightData.TxData.TxData.kommando   = CMD_BETTY_MASTER_DEMO;
   rfStarLightData.TxData.TxData.parameter1 = BETTY_MASTER_UNDEFINED;
   rfStarLightData.TxData.TxData.parameter2 = BETTY_MASTER_UNDEFINED;

   // Daten senden
   return_value = StarLightSend ( NOT_SHOW_SEND_POPUP );

   // Datenstrukturen zurcksetzen
   RfStarLightPurgeBuffer( );

   // Rckgabewert setzen
   return return_value;

}
//----- Ende RfStarLightDemomodus ---------------------------------------------


/* ----------------------------------------------------------------------------
Funktion: RfStarLightDimmerGesamt

Mit dieser Funktion knnen fr die Dimmer sowohl Status als auch Dimmwert auf
einmal gesetzt werdeb

bergabeparameter:      dimmer     = Dimmer [1..8] oder Broadcast
                        dimmstatus = Dimmer Ein/Aus [True/False]
                        dimmwert   = [1..31] Dimmwert

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightDimmerGesamt ( unsigned char dimmer , unsigned char dimmstatus , unsigned char dimmwert )
{
   /* ----- lokale Variablen ----- */
   unsigned char return_value = true;
   unsigned char temp_param;

   #if CMSSTARLIGHT_DEBUG
      char trace_string[255];
   #endif


   /* ----- Programmcode ----- */

   #if CMDSTARLIGHT_DEBUG
      serial_puts("Enter RfStarLightDimmerGesamt\n");
   #endif

   // bertragungsdaten in SendeFrame setzen
   rfStarLightData.TxData.TxData.kommando   = CMD_BETTY_MASTER_DIMMER_SCHALTEN;

   // Bemrkung: bei dieser Funktion bernimmt parameter1 Sonderrolle da
   //           High-Nibbel und Low-Nibbel unterschiedliche Bedeutungen haben

   // In Low-Nibble Dimmer oder Broadcast eintragen
   temp_param = dimmer & 0x0F;

   // Bit 8 setzen falls Funktion eingeschaltet werden soll, sonst aus
   if ( dimmstatus == STARLIGHT_FUNKTION_EIN )
   {
      temp_param |= 0x80;
   }
   rfStarLightData.TxData.TxData.parameter1 = temp_param;
   rfStarLightData.TxData.TxData.parameter2 = dimmwert;

   // Daten senden
   return_value = StarLightSend ( NOT_SHOW_SEND_POPUP );

   // Rckgabewert setzen
   return return_value;

}
//----- Ende RfStarLightDimmerGesamt ------------------------------------------


/* ============================ ENDE ======================================= */
