/**********************************************************************
@file       Rfstarligt.h

@brief      Interface for communication with SCART adapter via RF

            Modul bildet das Interface zwischen Betty und dem StarLight
            Master an welchem der SCART Adapter angebunden ist.


-----------------------------------------------------------------------
@author  2010 Markus Fulde
-----------------------------------------------------------------------
Following compile flags are usable:


-----------------------------------------------------------------------
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------------
@History

1.00 17092010    MaFu    Ersterstellung

@end
***********************************************************************/


#ifndef __RFSTARLIGHT_H__
#define __RFSTARLIGHT_H__


/* =============================== Includes ================================ */

#include "buffer.h"
#include "starlight.h"
#include "rf.h"


/* ================== Datentypen und Variablendefinitionen ================= */

// Kommandobytes fr Kommunikation BETTY->MASTER definieren
#define  CMD_BETTY_MASTER_STATUS          0x00
#define  CMD_BETTY_MASTER_ANLAGE          0x01
#define  CMD_BETTY_MASTER_DIMWERT         0x02
#define  CMD_BETTY_MASTER_SOUND           0x03
#define  CMD_BETTY_MASTER_LAMPEN          0x04
#define  CMD_BETTY_MASTER_DIMMER          0x05
#define  CMD_BETTY_MASTER_DEMO            0x06
#define  CMD_BETTY_MASTER_DIMMER_SCHALTEN 0x07

#define  BETTY_MASTER_BROADCAST           0xFF

#define  BETTY_MASTER_UNDEFINED           0xFF
#define  ACKNOWLEDGE                      0xFF

#define  SHOW_SEND_POPUP                  true
#define  NOT_SHOW_SEND_POPUP              false

typedef enum RfBinaryStatus_e
{
   RF_BINARY_IDLE,
   RF_BINARY_WAITING,
   RF_BINARY_TIMEOUT,
   RF_BINARY_ERROR,
   RF_BINARY_DATA_RECEIVED,
   RF_BINARY_MAX
} RfBinaryStatus_t;


#define RF_BINARY_FLAG_NONE                 0
#define RF_BINARY_FLAG_DATA_EXPECTED   _BV (0)
#define RF_BINARY_FLAG_TIMEOUT         _BV (1)


// [1] Framelnge
// [2] Kommando
// [3] Dimmer
// [4] Dimmwert
// [5] Temperatur
// [6] Sound
// [7] Checksumme Low-Byte
// [8] Checksumme High-Byte
#define RF_RX_MESSAGE_LENGTH 8

// [1] Framelnge
// [2] Kommando
// [3] Parameter 1
// [4] Parameter 2
// [5] Checksumme Low-Byte
// [6] Checksumme High-Byte
#define RF_TX_MESSAGE_LENGTH  6


// Datenstruktur zum Empfangen von Daten
typedef union RfStarLightRxData_s
{
   struct RxData_s {
      unsigned char  framelength;    // [1]
      unsigned char  kommando;       // [2]
      unsigned char  dimmer;         // [3]
      unsigned char  dimmwert;       // [4]
      unsigned char  temperatur;     // [5]
      unsigned char  sound;          // [6]
      unsigned char  checksumme_lb;  // [7]
      unsigned char  checksumme_hb;  // [8]
   } RxData;
   unsigned char     RxBuffer[ RF_RX_MESSAGE_LENGTH ];
} RfStarLightRxData_t, *RfStarLightRxData_p;


// Datenstruktur zum Senden von Daten
typedef union RfStarLightTxData_s
{
   // Strukt definieren und mit Inhalten fr Statusabfrage vorbereiten
   struct TxData_s {
      unsigned char  framelength;    // [1]
      unsigned char  kommando;       // [2]
      unsigned char  parameter1;     // [3]
      unsigned char  parameter2;     // [4]
      unsigned char  checksumme_lb;  // [5]
      unsigned char  checksumme_hb;  // [6]
   } TxData;
   unsigned char     TxBuffer[ RF_TX_MESSAGE_LENGTH ];
} RfStarLightTxData_t, *RfStarLightTxData_p;


// Gesamtdatenstruktur fr RfStarLight
typedef struct RfStarLightData
{
   RFendpoint_p          pRxEndpoint;
   RfStarLightRxData_t   RxData;

   RFendpoint_p          pTxEndpoint;
   RfStarLightTxData_t   TxData;

   volatile unsigned long timer;
   unsigned char   flags;

} RfStarLightData_t, *RfStarLightData_p;


/* ======================== Prototyping ==================================== */

/* ----------------------------------------------------------------------------
Funktion: RfStarLightInit

Funktion initialisiert zu Beginn bei Systemstart die Buffer zum Senden und
Empfangen von Daten via RF.

bergabeparameter:      keiner

Rckgabeparameter:      TRUE = Funktion war erfolgreich
---------------------------------------------------------------------------- */
BOOL RfStarLightInit ( void );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightOpen

Funktion erffnet eine Funkverbindung

bergabeparameter:      keiner

Rckgabeparameter:      TRUE  = Open  erfolgreich
                        FASLE = Open fehlgeschlagen
---------------------------------------------------------------------------- */
BOOL RfStarLightOpen ( void );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightSend

Funktion sendet den Sendebuffer via RF an den Master und empfngt die ent-
sprechende Antwort in dem Empfangbuffer.

bergabeparameter:      keiner

Rckgabeparameter:      TRUE  = Es wurden Daten emfpangen oder ein Timeout ist
                                aufgetreten.
                        FALSE = Funktion ohne Erfolg gelaufen

Bemerkung: Durch den Aufruf RfStarLightGetState kann abgefragt werden, on
           Daten empfangen wurden oder ein Timeout aufgetreten ist.
---------------------------------------------------------------------------- */
BOOL RfStarLightSend ( void );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightDone

Diese Funktion mu aufgerufen werden, wenn der Rckgabewert von RfStarLightSend
TRUE zurckliefert.
Wenn RfStarLightSend TRUE zurck liefert so wurden entweder Daten empfangen
oder ein Timeout ist abgelaufen.


bergabeparameter:      TRUE  = Es wird auf eine Antwort gewartet
                        FALSE = Keine Antwort erforderlich

Rckgabeparameter:      TRUE  = Es wurden Daten emfpangen oder ein Timeout ist
                                aufgetreten.
                        FALSE = Funktion ohne Erfolg gelaufen

Bemerkung: Durch den Aufruf RfStarLightGetState kann abgefragt werden, on
           Daten empfangen wurden oder ein Timeout aufgetreten ist.
---------------------------------------------------------------------------- */
// void RfStarLightDone ( void );


/* ----------------------------------------------------------------------------
Funktion: StarLightSend

Funktion sendet den Sendebuffer via RF an den Master und empfngt die ent-
sprechende Antwort in dem Empfangbuffer.

bergabeparameter:      popup_flag = gibt an ob Uhrenglas angezeigt werden soll
                                     True  = Ja
                                     False = Nein

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL StarLightSend ( unsigned char popup_flag );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightGetState

Diese Funktion gibt den aktuellen Status der internen Statemachine zurck

typedef enum RfBinaryStatus_e
{
   RF_BINARY_IDLE,
   RF_BINARY_WAITING,
   RF_BINARY_TIMEOUT,
   RF_BINARY_ERROR,
   RF_BINARY_DATA_RECEIVED,
   RF_BINARY_MAX
} RfBinaryStatus_t;

#define RF_BINARY_FLAG_NONE                 0
#define RF_BINARY_FLAG_DATA_EXPECTED   _BV (0)
#define RF_BINARY_FLAG_TIMEOUT         _BV (1)

bergabeparameter:      keiner

Rckgabeparameter:      siehe Beschreibung der Stati

Bemerkung: Durch den Aufruf RfStarLightGetState kann abgefragt werden, ob
           Daten empfangen wurden oder ein Timeout aufgetreten ist.
---------------------------------------------------------------------------- */
// RfBinaryStatus_t RfStarLightGetState ( void );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightStatusabfrage

Funktion dient zur Statusabfrage der Betriebsstati eines Dimmers.
Rckgabewerte sind:  [1] Kommando
                     [2] Dimmer
                     [3] Dimmwert
                     [4] Temperatur
                     [5] Sound

bergabeparameter:      dimmer = Dimmer [1..8]

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightStatusabfrage ( unsigned char dimmer );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightLichtanlageSchalten

Funktion ist Toggelfunktion und dient zum einschalten und ausschalten der
Lichtanlage.
Ist die Lichtanlage ausgeschaltet wird prinzipiell immer ACK mit ACK = 0
emfangen.

bergabeparameter:      keiner

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightLichtanlageSchalten ( void );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightDimmerSteuern

Mit dieser Funktion knnen die einzelnen Dimmer der Slaves ber den Master
angesteuert werden.

bergabeparameter:      dimmer   = Auswahl Dimmer [1..8]
                        dimmwert = Dimmwert [1..31]

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightDimmerSteuern ( unsigned char dimmer , unsigned char dimmwert );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightSoundSchalten

Funktion sendet die Sound-Einstellung zum Master und damit zu den Slaves.
ber diese Funktion wird eingestellt, ob Master und Slaves mit akustischem
Feedback bei Auswahl von Funktionen antworten.

bergabeparameter:      sound = Sound Ein/Aus [True/False]

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightSoundSchalten ( unsigned char sound );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightGesamtlicht

Mit dieser Funktion kann das gesamte Licht aller Dimmer auf einmal
geschaltet werden.

bergabeparameter:      lichtstatus = Licht Ein/Aus [True/False]

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightGesamtlicht ( unsigned char lichtstatus );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightDimmerSchalten

Mit dieser Funktion kann ein einzelner Dimmer aus und eingeschaltet werden.

bergabeparameter:      dimmer     = Dimmer [1..89
                        dimmstatus = Dimmer Ein/Aus [True/False]

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightDimmerSchalten ( unsigned char dimmer , unsigned char dimmstatus );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightPurgeBuffer

Funktion leert den Empfangsbuffer und bereitet Datenstrukt soweit notwendig auf
Empfang neuer Daten vor.

bergabeparameter:      keiner

Rckgabeparameter:      keiner
---------------------------------------------------------------------------- */
void RfStarLightPurgeBuffer ( void );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightDemomodus

Funktion lst im Master den Demomodus aus.

bergabeparameter:      keiner

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightDemomodus ( void );


/* ----------------------------------------------------------------------------
Funktion: RfStarLightDimmerGesamt

Mit dieser Funktion knnen fr die Dimmer sowohl Status als auch Dimmwert auf
einmal gesetzt werdeb

bergabeparameter:      dimmer     = Dimmer [1..8] oder Broadcast
                        dimmstatus = Dimmer Ein/Aus [True/False]
                        dimmwert   = [1..31] Dimmwert

Rckgabeparameter:      TRUE = Funktion war erfolgreich
                        FALSE = Fehler bei der Kommunikation
---------------------------------------------------------------------------- */
BOOL RfStarLightDimmerGesamt ( unsigned char dimmer , unsigned char dimmstatus , unsigned char dimmwert );


#endif   // #ifndef __RFSTARLIGHT_H__


/* ============================ ENDE ======================================= */
